package com.FranckBarbier.Java._Fibonacci;

public class Fibonacci {

    static long _F = 0L;

    public enum Computation_type {

        Recursive, Iterative, Binet
    }
    public final double Golden_number = (1. + Math.sqrt(5.)) / 2.;
    public final double result;
    public long cost;

    public Fibonacci(long n, Computation_type ct) {
        switch (ct) {
            case Recursive:
                result = recursive(n);
                break;
            case Iterative:
                result = iterative(n);
                break;
            case Binet:
                result = binet(n);
                break;
            default:
                throw new EnumConstantNotPresentException(Computation_type.class, ct.name());
        }
        cost = _F;
        _F = 0L;
    }

    public long recursive(long n) {
        assert (n >= 0L);
        if (n == 0L) {
            return 0L;
        }
        if (n == 1L) {
            return 1L;
        }
        _F++;
        return recursive(n - 1L) + recursive(n - 2L);
    }

    public long iterative(long n) {
        assert (n >= 0L);
        if (n == 0L) {
            return 0L;
        }
        if (n == 1L) {
            return 1L;
        }
        long u = 1L, v = 0L, u0, v0;
        for (long i = 2L; i <= n; i++) {
            u0 = u;
            v0 = v;
            u = u0 + v0;
            v = u0;
        }
        return u;
    }

    long binet(long n) {
        assert (n >= 0L);
        return (long) ((Math.pow(Golden_number, (double) n) - Math.pow((1. - Golden_number), (double) n)) / Math.sqrt(5.));
    }

    public static void main(String[] args) {
        java.util.Date start = new java.util.Date();
        Fibonacci fibonacci = new Fibonacci(40L, Fibonacci.Computation_type.Recursive);
        java.util.Date end = new java.util.Date();
        System.out.print("Time spent in ms: " + (end.getTime() - start.getTime()));
        System.out.print("\tResult: " + fibonacci.result);
        System.out.print("\tCost: " + fibonacci.cost);
    }
}
